package com.qs.mp.web.controller.api.channel.mall;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qs.mp.admin.domain.TicketAwardsPrize;
import com.qs.mp.admin.domain.TicketBox;
import com.qs.mp.admin.domain.vo.TicketBoxListVO;
import com.qs.mp.admin.domain.param.TicketBoxParam;
import com.qs.mp.admin.domain.vo.TicketBoxVO;
import com.qs.mp.admin.service.ITicketAwardsPrizeService;
import com.qs.mp.admin.service.ITicketBoxService;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.core.page.TableDataInfo;
import com.qs.mp.common.enums.TicketBoxStatusEnum;
import com.qs.mp.common.utils.StringUtils;
import com.qs.mp.web.controller.common.BaseApiController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.List;
import lombok.AllArgsConstructor;
import ma.glasnost.orika.MapperFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


@RestController
@RequestMapping("/api/v1/mp/channel/mall")
@Api(tags = "渠道盲票商城接口")
@AllArgsConstructor
public class ChannelTicketController extends BaseApiController {


  @Autowired
  private ITicketBoxService ticketBoxService;

  @Autowired
  private ITicketAwardsPrizeService ticketAwardsPrizeService;

  @Autowired
  private MapperFacade mapperFacade;

  /**
   * 盲票进货列表
   */
  @PostMapping("/ticket/list")
  @ApiOperation(value = "盲票组列表" , notes = "获取所有盲票信息")
  public TableDataInfo list(@RequestBody TicketBoxParam param) {
    // TODO 根据标签过滤
    startPage();
    List<TicketBox> ticketBoxes = ticketBoxService.list(
        new LambdaQueryWrapper<TicketBox>().eq(TicketBox::getType, param.getType())
            .eq(null != param.getCategoryId() && 0 != param.getCategoryId(), TicketBox::getCategoryId, param.getCategoryId())
            .eq(TicketBox::getStatus, TicketBoxStatusEnum.PUT_ON));
    List<TicketBoxListVO> ticketBoxListVOList = mapperFacade.mapAsList(ticketBoxes, TicketBoxListVO.class);
    TableDataInfo rspData = getDataTable(ticketBoxes);
    rspData.setRows(ticketBoxListVOList);
    return rspData;
  }


  /**
   * 查看盲票详情
   */
  @PostMapping("/ticket/detail")
  @ApiOperation(value = "查看盲票详情" , notes = "根据盲票组ID，获取盲票信息")
  public AjaxResult getInfo(@RequestBody TicketBoxParam param) {
    if (StringUtils.isBlank(param.getBoxId())) {
      return AjaxResult.error("参数异常，盲票组ID缺失");
    }
    TicketBox ticketBox = ticketBoxService.getById(param.getBoxId());
    TicketBoxVO ticketBoxVO = mapperFacade.map(ticketBox, TicketBoxVO.class);
    ticketBoxVO.setPrizeList(ticketAwardsPrizeService.listPrizeVO(new QueryWrapper<TicketAwardsPrize>()
        .eq("t1.box_id", ticketBox.getBoxId()).orderByAsc("t1.sort").orderByDesc("t2.value")));
    return AjaxResult.success(ticketBoxVO);
  }

}
