package com.qs.mp.web.controller.api.common;

import cn.jsms.api.ValidSMSResult;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.qs.mp.channel.domain.Channel;
import com.qs.mp.channel.service.IChannelService;
import com.qs.mp.common.constant.Constants;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.enums.UserIdentityEnum;
import com.qs.mp.common.enums.WxActTypeEnum;
import com.qs.mp.common.jsms.JSMSUtils;
import com.qs.mp.common.qcloud.QcloudFileUtils;
import com.qs.mp.common.utils.LogUtil;
import com.qs.mp.common.utils.StringUtils;
import com.qs.mp.common.utils.UUIDUtils;
import com.qs.mp.common.utils.http.HttpUtils;
import com.qs.mp.core.domain.LoginUser;
import com.qs.mp.framework.domain.WxAccount;
import com.qs.mp.framework.security.handle.HostHolder;
import com.qs.mp.framework.service.IAppTokenService;
import com.qs.mp.framework.service.IWxAccountService;
import com.qs.mp.framework.web.service.SysLoginService;
import com.qs.mp.system.domain.SysUser;
import com.qs.mp.system.domain.vo.WxLoginParams;
import com.qs.mp.system.service.ISysUserService;
import com.qs.mp.utils.SecurityUtils;
import com.qs.mp.web.controller.common.BaseApiController;
import io.swagger.annotations.Api;
import java.io.File;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * @auther duota
 * @create 2021 2021/9/6 6:16 下午
 * @describe
 */

@Api("用户钱包API")
@RestController
@RequestMapping("/api/v1/mp")
@Component
public class UserController extends BaseApiController {

  @Autowired
  private ISysUserService sysUserService;

  @Autowired
  protected HostHolder hostHolder;

  @Autowired
  private IWxAccountService wxAccountService;

  @Autowired
  private IChannelService channelService;

  @Value("${wx-channel.appId}")
  private String channelAppId;
  @Value("${wx-channel.appSecret}")
  private String channelAppSecret;

  @Value("${wx-user.appId}")
  private String userAppId;
  @Value("${wx-user.appSecret}")
  private String userAppSecret;

  /**
   * 文件上传路径
   */
  @Value("${mp.profile}")
  public String filePath;

  /**
   * 公开
   */
  @Value("${cloud.public-bucket-name}")
  private String publicBucketName;

  @Autowired
  private IAppTokenService appTokenService;

  @Autowired
  private SysLoginService sysLoginService;


  @RequestMapping(value = "/user/wxauth", method = RequestMethod.POST)
  @ResponseBody
  public AjaxResult wxauth(@RequestBody WxLoginParams wxLoginParams) {
    String appId = channelAppId;
    String appSecret = channelAppSecret;
    if (wxLoginParams.getIdentity() == UserIdentityEnum.USER.ordinal()) {
      appId = userAppId;
      appSecret = userAppSecret;
    }
    //调用微信后台接口获取openId
    String res = wxLoginCheck(wxLoginParams, appId, appSecret);
    JSONObject jsonObject = JSONObject.parseObject(res);
    String openId = jsonObject.getString("openid");
    String sessionKey = jsonObject.getString("session_key");
    String unionId = jsonObject.getString("unionid");

    if (StringUtils.isBlank(openId) || StringUtils.isBlank(sessionKey)) {
      LogUtil.error(logger, "微信登录失败:" + jsonObject.toJSONString());
      return error("登录失败");
    }
    //更新数据库用户授权信息
    LoginUser loginUser = SecurityUtils.getLoginUser();
    SysUser user = new SysUser();
    user.setUserId(loginUser.getUserId());
    if (wxLoginParams.getIdentity() == UserIdentityEnum.USER.ordinal()) {
      user.setOpenId(openId);
    } else {
      user.setChannelOpenId(openId);
    }

    user.setSessionKey(sessionKey);
    user.setUnionId(unionId);
    user.setLoginDate(new Date());
    //如果用户头像字段为空，则默认获取用户微信头像
    String avatarUrl = wxLoginParams.getAvatarUrl();

    //用户先关联过公众号，那么授权后进行自动绑定
    LambdaQueryWrapper<WxAccount> wrapper = new LambdaQueryWrapper();
    wrapper.eq(WxAccount::getUnionid,unionId);
    wrapper.eq(WxAccount::getActType, WxActTypeEnum.GZH.getValue());
    List<WxAccount> accountList = wxAccountService.list(wrapper);
    if(accountList.size() == 1){
      user.setGzhOpenId(accountList.get(0).getOpenid());
    }
    if(StringUtils.isBlank(loginUser.getUser().getAvatar()) && !StringUtils.isBlank(avatarUrl)){
      try {
        //下载并且重新上传头像
        JSONObject result = saveHeadImage(avatarUrl,String.valueOf(user.getUserId()));
        if(result != null){
          user.setAvatar(result.getString("fileName"));
        }
      } catch (Exception e) {
        LogUtil.error(logger,"头像上传异常",e.getMessage());
      }
    }
    logger.info("update user data:"+JSONObject.toJSONString(user));
    int rows = sysUserService.updateUserProfile(user);
    if (rows != 1) {
      LogUtil.error(logger,"用户微信授权异常");
      return error("用户微信授权异常");
    }
    return success("微信授权成功");
  }


  @RequestMapping(value = "/user/wxauth/mobile", method = RequestMethod.POST)
  @ResponseBody
  public AjaxResult wxauthMobile(@RequestBody WxLoginParams wxLoginParams) {
    String appId = userAppId;
    String appSecret = userAppSecret;
    if (UserIdentityEnum.isChannel(wxLoginParams.getIdentity())) {
      appId = channelAppId;
      appSecret = channelAppSecret;
    }
    //调用微信后台接口获取openId
    String res = getWxMobile(wxLoginParams, appId);
    JSONObject jsonObject = JSONObject.parseObject(res);
    int errcode = jsonObject.getIntValue("errcode");
    if (errcode != 0 ) {
      LogUtil.error(logger, "手机号获取失败:" + res);
      return error("手机号获取失败");
    }
    JSONObject phoneInfo = jsonObject.getJSONObject("phone_info");
    String phoneNumber = phoneInfo.getString("phoneNumber");

    if (StringUtils.isBlank(phoneNumber)) {
      LogUtil.error(logger, "微信手机号不存在:" + res);
      return error("手机号获取失败");
    }
    AjaxResult ajax = AjaxResult.success();

    SysUser sysUser = sysUserService.selectUserByUserName(phoneNumber);
    if (null == sysUser) {
      if (wxLoginParams.getIdentity() != UserIdentityEnum.USER.ordinal()) {
        return error("用户不存在，请联系客服");
      }
      // C端直接注册新用户
      sysUser = new SysUser();
      sysUser.setUserName(phoneNumber);
      sysUser.setNickName(phoneNumber);
      sysUserService.registerUser(sysUser);
    }
    Map<String, String> result = sysLoginService.wxAuthLogin(phoneNumber, wxLoginParams.getIdentity());
    for (String key : result.keySet()) {
      ajax.put(key, result.get(key));
    }
    return ajax;
  }

  /**
   * 重置密码
   */
  @RequestMapping(value = "/user/resetPwd", method = RequestMethod.POST)
  public AjaxResult resetPwd(@RequestBody JSONObject user) {
    //验证短信验证码
    ValidSMSResult res = JSMSUtils.sendValidSMSCode(
        user.getString("messageId"), user.getString("code"));
    if (res == null || !res.getIsValid()) {
      return error("短信验证码错误");
    }

    int rows = sysUserService.resetUserPwd(user.getString("userName"),
        SecurityUtils.encryptPassword(user.getString("password")));
    if (rows == 1) {
      return success("密码修改成功.");
    } else {
      return error("密码修改失败");
    }
  }

  /**
   * 查询当前登录用户信息
   */
  @RequestMapping(value = "/user/getLoginUserinfo", method = RequestMethod.POST)
  public AjaxResult getLoginUserinfo(@RequestBody JSONObject params) {
    LoginUser loginUser = SecurityUtils.getLoginUser();

    JSONObject jsonObject = new JSONObject();
    jsonObject.put("userName", loginUser.getUsername());
    jsonObject.put("avatar", loginUser.getUser().getAvatar());
    if(loginUser.getIdentity() == UserIdentityEnum.CHANNEL) {
      Channel channel = channelService.getById(loginUser.getChannelId());
      jsonObject.put("nickName", channel.getName());
    } else {
      jsonObject.put("nickName", loginUser.getUser().getNickName());
    }
    jsonObject.put("openId", loginUser.getUser().getOpenId());
    jsonObject.put("channelOpenId", loginUser.getUser().getChannelOpenId());
    JSONArray roleKeys = new JSONArray();
    JSONArray roles = new JSONArray();
    loginUser.getUser().getRoles().forEach(sysRole -> roles.add(sysRole.getRoleName()));
    jsonObject.put("roleKeys", roleKeys);
    jsonObject.put("roles", roles);

    return success(jsonObject);
  }



  /**
   * code 换取 session_key
   *
   * @param
   * @return
   */
  private String wxLoginCheck(WxLoginParams params, String appId, String appSecret) {
    String param = "appid=" + appId + "&secret=" + appSecret + "&js_code=" + params.getCode()
        + "&grant_type=authorization_code";
    String res = HttpUtils.sendGet("https://api.weixin.qq.com/sns/jscode2session", param);
    return res;
  }

  /**
   * code 换取 手机号
   *{
   *     "errcode":0,
   *     "errmsg":"ok",
   *     "phone_info": {
   *         "phoneNumber":"xxxxxx",
   *         "purePhoneNumber": "xxxxxx",
   *         "countryCode": 86,
   *         "watermark": {
   *             "timestamp": 1637744274,
   *             "appid": "xxxx"
   *         }
   *     }
   * }
   * @param
   * @return
   */
  private String getWxMobile(WxLoginParams params, String appId) {
    String accessToken = appTokenService.getAccessToken(appId);
    JSONObject jsonObject = new JSONObject();
    jsonObject.put("code", params.getCode());
    LogUtil.info(logger, JSONObject.toJSONString(params));
    String res = HttpUtils.sendPost("https://api.weixin.qq.com/wxa/business/getuserphonenumber?access_token=" + accessToken, jsonObject.toJSONString());
    return res;
  }

  /**
   * { "subscribe": 1, "openid": "o6_bmjrPTlm6_2sgVt7hMZOPfL2M", "nickname": "Band", "sex": 1,
   * "language": "zh_CN", "city": "广州", "province": "广东", "country": "中国", "headimgurl":
   * "http://wx.qlogo.cn/mmopen/g3MonUZtNHkdmzicIlibx6iaFqAc56vxLSUfpb6n5WKSYVY0ChQKkiaJSgQ1dZuTOgvLLrhJbERQQ4eMsv84eavHiaiceqxibJxCfHe/0",
   * "subscribe_time": 1382694957, "unionid": " o6_bmasdasdsad6_2sgVt7hMZOPfL" }
   *
   * @param accessToken
   * @param openId
   * @return
   */
  private String getWxUserInfo(String accessToken, String openId) {
    String url = "https://api.weixin.qq.com/cgi-bin/user/info";
    String params = "access_token=" + accessToken + "&openid=" + openId + "&lang=zh_CN";
    String res = HttpUtils.sendGet(url, params);
    logger.info("weixin user info :" + res);
    return res;
  }

  private JSONObject saveHeadImage(String serverUrl,String userId) throws Exception {
    String outFilePath = filePath+"/headImage";
    File outPath = new File(outFilePath);
    if (!outPath.exists()) {
      if (!outPath.mkdirs()) {
        throw new Exception("makdirs: '" + outFilePath + "'fail");
      }
    }
    URL url = new URL(serverUrl);
    HttpURLConnection conn = (HttpURLConnection) url.openConnection();
    conn.setConnectTimeout(3 * 1000);
    //防止屏蔽程序抓取而放回403错误
    conn.setRequestProperty("User-Agent", "Mozilla/4.0(compatible;MSIE 5.0;Windows NT;DigExt)");
    Long totalSize = Long.parseLong(conn.getHeaderField("Content-Length"));
    String name = userId+"/"+ UUIDUtils.newId();
    String mimeType = "image/jpg";
    //logger.info("图片开始下载. totalSize" + totalSize);
    if (totalSize > 0) {
      File file = new File(outFilePath+"/"+userId);
      FileUtils.copyURLToFile(url, file);
      //logger.info("图片下载完成.");
      QcloudFileUtils.putFile(file, name, publicBucketName, mimeType );
      FileUtils.delete(file);
    } else {
      throw new Exception("can not find serverUrl :{}" + serverUrl);
    }
    JSONObject result = new JSONObject();
    result.put("fileName",name);
    result.put("fileType",mimeType);
    return result;
  }

}
